"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("../mongoose/model-config");
const _ = require("underscore");
const debug_1 = require("debug");
const d = debug_1.default('sbase:overrides');
const object_path_1 = require("object-path");
function overrides(...rules) {
    const rs = [];
    for (const rule of rules) {
        if (_.isString(rule)) {
            const [os, od] = rule.split('->');
            if (!od) {
                throw new Error(`Rule ${rule} is not correct`);
            }
            rs.push({ src: split(os), dst: split(od) });
        }
        else {
            const [os, od] = rule;
            rs.push({ src: _.isFunction(os) ? os : () => os, dst: split(od) });
        }
    }
    d('Prepared overrides: %O', rs);
    return async (ctx, next) => {
        for (const { src, dst } of rs) {
            let value;
            if (_.isArray(src)) {
                value = object_path_1.withInheritedProps.get(ctx, src);
            }
            else if (_.isFunction(src)) {
                value = src(ctx);
                if (value && value.then) {
                    value = await value;
                }
            }
            if (value !== undefined) {
                object_path_1.withInheritedProps.set(ctx, ['overrides'].concat(dst), value);
                d('Set overrides %O with value %O, result: %O', dst, value, ctx.overrides);
            }
        }
        await next();
    };
}
exports.overrides = overrides;
function clearOverrides() {
    return async (ctx, next) => {
        ctx.overrides = {
            query: {},
            doc: {},
        };
        await next();
    };
}
exports.clearOverrides = clearOverrides;
function split(str) {
    const strs = str.split('.');
    for (let i = strs.length - 1; i >= 1; i--) {
        if (strs[i - 1].endsWith('\\')) {
            strs[i - 1] =
                strs[i - 1].substring(0, strs[i - 1].length - 1) + '.' + strs[i];
            strs[i] = '';
        }
    }
    return _.filter(strs, x => !!x);
}

//# sourceMappingURL=overrides.js.map
