import * as model from './model';
import { IMiddleware, IRouterContext } from 'koa-router';
import { ModelPopulateOptions } from 'mongoose';
export declare const READONLY = "READONLY";
export declare const AUTOGEN = "AUTOGEN";
export declare type KoaMiddlewaresType = typeof KoaMiddlewares;
export declare class KoaMiddlewares extends model.DocumentModel {
    static createMiddleware(options: CreateOptions): IMiddleware;
    /**
     * Returns Koa get middleware.
     */
    static getMiddleware(options: GetOptions): IMiddleware;
    static findMiddleware(options: FindOptions): IMiddleware;
    static updateMiddleware(options: UpdateOptions): IMiddleware;
    static deleteMiddleware(options: DeleteOptions): IMiddleware;
}
export interface CommonOptions {
    noBody?: boolean;
    triggerNext?: boolean;
    target?: string;
    lean?: boolean;
    transform?: (a: any, ctx: IRouterContext) => Promise<any>;
}
export interface CommonResponseOptions {
    level?: string;
    project?: string[];
    populate?: ModelPopulateOptions | ModelPopulateOptions[];
}
export interface CommonReadOptions {
}
export interface CommonWriteOptions {
    omits?: string[];
}
export interface CreateOptions extends CommonOptions, CommonResponseOptions, CommonWriteOptions {
}
/**
 * When read/write/delete to a single item.
 */
export interface SingleItemOptions {
    field: string;
    idFieldName?: string;
    nullable?: boolean;
}
/**
 * Get Middleware Options.
 */
export interface GetOptions extends CommonOptions, CommonResponseOptions, CommonReadOptions, SingleItemOptions {
}
export interface FindOptions extends CommonOptions, CommonResponseOptions, CommonReadOptions {
    pagination?: {
        size?: number;
        sizeChoices?: number[];
    };
    sort?: object;
}
export interface UpdateOptions extends CommonOptions, CommonResponseOptions, CommonWriteOptions, SingleItemOptions {
}
export interface DeleteOptions extends CommonOptions, SingleItemOptions {
}
export declare type INext = () => Promise<any>;
export declare function Autogen(schema?: any): PropertyDecorator;
export declare function Readonly(schema?: any): PropertyDecorator;
